#############
# $Id: SecondBaseCall.pm 13506 2011-07-01 09:19:27Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2010-11-18
# Last Changed On: $Date: 2011-07-01 10:19:27 +0100 (Fri, 01 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-28.0/lib/npg_pipeline/analysis/SecondBaseCall.pm $

package npg_pipeline::analysis::SecondBaseCall;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13506 $ =~ /(\d+)/mxs; $r; };
use Cwd;      # for getcwd

Readonly::Scalar our $NUMBER_OF_READS_INDEXED_AND_PAIRED  => 3;
Readonly::Scalar our $NUMBER_OF_READS_INDEXED_OR_PAIRED   => 2;
Readonly::Scalar our $MINIMUM_NUMBER_OF_READS             => 1;
Readonly::Scalar our $SECOND_LITERAL_READ_NUMBER          => 2;
Readonly::Scalar our $FIRST_READ_CYCLE_DEFAULT            => 1;

extends q{npg_pipeline::base};
with q{npg_common::roles::run::long_info};
with q{npg_common::roles::run::fs_resource};
with q{npg_pipeline::roles::business::bustard_lsf_reqs};
with q{npg_pipeline::roles::business::harold_calibration_reqs};
with q{npg_common::roles::run::lane::tag_info};

=head1 NAME

npg_pipeline::analysis::SecondBaseCall

=head1 VERSION

$LastChangedRevision: 13506 $

=head1 SYNOPSIS

  use npg_pipeline::analysis::SecondBaseCall;
  my $SecondBaseCall = npg_pipeline::analysis::SecondBaseCall->new( {
    id_run => $iIdRun,
  } );

=head1 DESCRIPTION

This module handles launching a script to run generation of second basecall qseq files.

=head1 SUBROUTINES/METHODS

=head2 submit_to_lsf

The method to call, which will generate and submit the script to generate the second call.

=cut

sub submit_to_lsf {
  my ( $self, $arg_refs ) = @_;

  my @job_ids;

  my $basecalls_dir = $self->intensity_path() . q{/BaseCalls};

  my $secondcall_dir = $self->bustard_path() . q{/SecondCall};
  $self->make_log_dir( $secondcall_dir );

  my $cwd = getcwd();

  chdir $secondcall_dir or croak qq{could not cd to $secondcall_dir: $ERRNO};

  $self->log( qq{Changed to $secondcall_dir to launch job.} );

  my $max_read = $self->is_paired_read() && $self->is_indexed() ? $NUMBER_OF_READS_INDEXED_AND_PAIRED
               : $self->is_paired_read() || $self->is_indexed() ? $NUMBER_OF_READS_INDEXED_OR_PAIRED
               :                                                  $MINIMUM_NUMBER_OF_READS
               ;

  foreach my $read_number ( 1..$max_read ) {
    $arg_refs->{read_number} = $read_number;
    push @job_ids, $self->submit_bsub_command( $self->_generate_bsub_command( $arg_refs ) );
  }

  chdir $cwd or croak qq{could not cd back to $cwd: $ERRNO};

  $self->log( qq{Changed back to $cwd.} );

  return @job_ids;
}


############
# private methods

sub _generate_bsub_command {
  my ( $self, $arg_refs ) = @_;

  my $id_run = $self->id_run();
  my $required_job_completion = $arg_refs->{required_job_completion};

  my $job_name = q{second_basecall_} . $id_run . q{_} . $self->timestamp();
  my $fs_resource_string = $self->fs_resource_string( {
    counter_slots_per_job => 1,
  } );

  my $array_string = $self->create_array_string( $self->positions( $arg_refs ) ) || q{};

  my $cmd = q{bsub -q } . $self->lsf_queue()
          . q{ -J '} . $job_name . $array_string . q{'}
          . q{ } . $fs_resource_string
          . q{ } . $required_job_completion
          . q{ -o } . $self->make_log_dir( $self->bustard_path() . q{/SecondCall} ) . qq{/$job_name.} . q{%I.%J.out}
          . q{ '} . $self->pb_calibration_bin() . q{/} . $self->pb_cal_pipeline_conf()->{second_basecall_script}
          . q{ -bustard_dir } . $self->bustard_path()
          . q{ -runfolder_dir } . $self->runfolder_path()
          . q{ -run } . $id_run
          . q{ -lane } . $self->lsb_jobindex()
          . q{ -read } . $arg_refs->{read_number};

  my $cycle_start;
  if ($arg_refs->{read_number} == $MINIMUM_NUMBER_OF_READS) {
    my @range = $self->read1_cycle_range();
    $cycle_start = $range[0];
  } else {
    if ( $self->is_indexed() && $arg_refs->{read_number} == $SECOND_LITERAL_READ_NUMBER ) {
      my @range = $self->indexing_cycle_range();
      $cycle_start = $range[0];
    } else {
      my @range = $self->read2_cycle_range();
      $cycle_start = $range[0];
    }
  }

  $cmd .= q{ -cstart } . $cycle_start;

  if ( $self->instrument_type eq q{HiSeq} ) {
    $cmd .= q{ -scl};
  }

  $cmd .= q{'};

  if ( $self->verbose() ) {
    $self->log( $cmd );
  }

  return $cmd;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
