#############
# $Id: fix_qseqs.pm 13506 2011-07-01 09:19:27Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-10-23
# Last Changed On: $Date: 2011-07-01 10:19:27 +0100 (Fri, 01 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-28.0/lib/npg_pipeline/archive/file/fix_qseqs.pm $

package npg_pipeline::archive::file::fix_qseqs;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13506 $ =~ /(\d+)/mxs; $r; };

extends qw{npg_pipeline::archive::file::generation};

Readonly::Scalar our $FIX_QSEQ_RUN_ID_SCRIPT => q{fix_qseq_run_id};

# this is probably going to remain redundant, so these will stay as variables specific to this,
# and so not be added to a conf file
Readonly::Scalar our $TIME_DELAY_SECONDS => 60;
Readonly::Scalar our $TIME_DELAY_MINUTES => 6;

sub fix_qseqs {
  my ($self, $arg_refs) = @_;
  my $id_run = $arg_refs->{id_run};

  $self->log(qq{Creating Jobs to ensure qseq files have correct id_run - $id_run});

  my @job_ids;

  foreach my $lane ($self->_lanes()) {
    $arg_refs->{lane} = $lane;
    my $job_sub = $self->_generate_bsub_command($arg_refs);
    if ($job_sub) { # only submit a job if there is a job submission returned
      push @job_ids, $self->submit_bsub_command($job_sub);
    }
  }
  return @job_ids;
}

###############
# this will be submitted once for each lane, so obtain lanes to work obtain the qseq file names
sub _lanes {
  my ($self) = @_;

  my @lanes;

  my $run = $self->run();
  if ($run) {
    foreach my $lane (@{$run->run_lanes()}) {
      push @lanes, $lane->position();
    }
  } else {
    @lanes = (qw{1 2 3 4 5 6 7 8});
  }

  return @lanes;
}


###############
# responsible for generating the bsub command to be executed for non_indexed runs
sub _generate_bsub_command {
  my ($self, $arg_refs) = @_;

  my $lane = $arg_refs->{lane};
  my $qseq_file_string = $self->qseq_file_constructs($lane);

  # only create a bsub command if there are any qseq_files found
  if (!$qseq_file_string) {
    return;
  }

  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $arg_refs->{timestamp};
  my $solexa_bin = $self->internal_script_bin();
  my $id_run = $arg_refs->{id_run};

  my $outfile = $self->recalibrated_path().q{/}.$arg_refs->{BSUB_OUTPUT_FILE};
  my $errfile = $self->recalibrated_path().q{/}.$arg_refs->{BSUB_ERROR_FILE};
  my $replacement = $FIX_QSEQ_RUN_ID_SCRIPT . q{_} . $lane   .q{_} . $timestamp;
  my $job_name    = $FIX_QSEQ_RUN_ID_SCRIPT . q{_} . $id_run .q{_} . $lane . q{_}.$timestamp;
  $outfile =~ s/substitute_this/$replacement/xms;
  $errfile =~ s/substitute_this/$replacement/xms;

  # -b time + 6minutes added as emergency hack to ensure this script completes before qseq fix runs - this script can be reliant on dir contents being contant
  my $job_sub =  q{bsub -b }.(join q{:},(localtime time+$TIME_DELAY_SECONDS*$TIME_DELAY_MINUTES )[2,1])
              .  q{ -q } . $self->lsf_queue() . qq{ $required_job_completion -J $job_name }
              . ( $self->fs_resource_string( {
                    counter_slots_per_job => 1,
                  } ) )
              . qq{ -o $outfile '$solexa_bin/$FIX_QSEQ_RUN_ID_SCRIPT -id_run=$id_run $qseq_file_string'};

  if ($self->verbose()) { $self->log($job_sub); }
  return $job_sub;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::fix_qseqs

=head1 VERSION

$LastChangedRevision: 13506 $

=head1 SYNOPSIS

  my $oFixQseqs = npg_pipeline::archive::file::fix_qseqs->new({
      run_folder => <run_folder>,
      analysis_path => <analysis_path>,
      id_run => <id_run>,
    });

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fix_qseqs - method to call which will go away and submit the jobs, returning the job ids submitted

  my @job_ids = $oFixQseqs->fix_qseqs({
      required_job_completion  => $sJobCompletion,
      timestamp                => $sTimestamp,
      id_run                   => $iIdRun,
      BSUB_ERROR_FILE          => $sBSUB_ERROR_FILE,
      BSUB_OUTPUT_FILE         => $sBSUB_OUTPUT_FILE,
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
