#############
# $Id: qc.pm 13506 2011-07-01 09:19:27Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-07-30
# Last Changed On: $Date: 2011-07-01 10:19:27 +0100 (Fri, 01 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-28.0/lib/npg_pipeline/archive/file/qc.pm $

package npg_pipeline::archive::file::qc;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13506 $ =~ /(\d+)/mxs; $r; };
use File::Spec;
use List::MoreUtils qw{any};

use npg_qc::autoqc::autoqc;

extends qw{npg_pipeline::archive::file::generation};
with qw{
  npg_common::roles::run::lane::tag_info
  npg_pipeline::roles::business::pre_exec_strings
};

Readonly::Scalar our $INSERT_SIZE_FARM_OPTIONS => q{-R 'select[type==X86_64,mem>6000] rusage[mem=6000]' -M6000000};
Readonly::Scalar our $LANE_MULTIPLIER_FACTOR_FOR_ARRAY_INDEX => 1000;
Readonly::Scalar our $NUMBER_OF_READS_INDEXED_AND_PAIRED => 3;
Readonly::Scalar our $NUMBER_OF_READS_INDEXED_OR_PAIRED => 2;
Readonly::Scalar our $MINIMUM_NUMBER_OF_READS => 1;
Readonly::Scalar our $FILTERED_OUT_SPIKED_PHIX => q[spiked_phix_sequence];

Readonly::Scalar our $NO_REFERENCES_OR_ADAPTER_REQUIREMENTS => {
  qX_yield => 1,
};


has q{qc_to_run}    => (isa => q{Str}, is => q{ro}, required => 1);
has q{farm_options} => (isa => q{Str}, is => q{rw}, default => q{});

has q{_lanes_by_split_nonsplit} => (
  isa => q{HashRef},
  is  => q{rw},
  lazy_build => 1,
);

sub _build__lanes_by_split_nonsplit {
  my ( $self ) = @_;
  return $self->lanes_by_split_nonsplit( $self->positions() );
}

sub run_qc {
  my ($self, $arg_refs) = @_;

  my $qc_to_run = $self->qc_to_run();
  if ($qc_to_run =~ /insert_size|sequence_error|ref_match/smx ) {
    $self->farm_options($INSERT_SIZE_FARM_OPTIONS);
  }

  $self->farm_options( $self->fs_resource_string( {
    resource_string => $self->farm_options(),
    counter_slots_per_job => 1,
  } ) );

  my $id_run = $self->id_run();
  $self->log(qq{Running qc test $qc_to_run on Run $id_run});

  my @lanes = $self->positions($arg_refs);

  my @positions_to_run_on;

  my @job_ids;

  foreach my $position (@lanes) {

    if ($arg_refs->{$FILTERED_OUT_SPIKED_PHIX}) {
      if ( $self->_is_lane_spiked_phix($position) eq q{true} ) {
        push @positions_to_run_on, $position;
      } else {
        next;
      }
    } else {
      # check to see if this test should run on this lane, don't submit a job for it if it shouldn't
      if ( $self->_can_run($position) ) {
        push @positions_to_run_on, $position;
      }

      #######
      # qc for individual plex
      #
      if ( $self->is_indexed && (-e $self->lane_archive_path( $position ) ) ) {
        my $lane_qc_dir = $self->lane_qc_path( $position );
        if (!-e $lane_qc_dir) {
          mkdir $lane_qc_dir;
        }
      }
    }
  }

  if ( scalar @positions_to_run_on ) {
    $arg_refs->{array_string} = $self->create_array_string( @positions_to_run_on );
    my $bsub_command = $self->_generate_bsub_command( $arg_refs );
    if ( $bsub_command ) {
      push @job_ids, $self->submit_bsub_command( $self->_generate_bsub_command( $arg_refs ) );
    }
  }

  if ( ! $arg_refs->{$FILTERED_OUT_SPIKED_PHIX} && $self->is_indexed() ) {
    my $bsub_command = $self->_generate_bsub_command( $arg_refs, 1 );
    if ( $bsub_command ) {
      push @job_ids, $self->submit_bsub_command( $bsub_command );
    }
  }

  return @job_ids;
}


sub _generate_bsub_command {
  my ($self, $arg_refs, $indexed) = @_;

  my $array_string = $arg_refs->{array_string};
  my $required_job_completion = $arg_refs->{required_job_completion} || q{};
  my $timestamp = $self->timestamp();
  my $solexa_bin = $self->internal_script_bin();
  my $id_run = $self->id_run();
  my $qc_variations = $arg_refs->{$FILTERED_OUT_SPIKED_PHIX} ? q{_spiked_phix} : q{};

  my $job_name = $self->external_script_names_conf()->{qc_script} . q{_} . $self->qc_to_run() . $qc_variations;
  $job_name .= q{_} . $id_run . q{_} . $timestamp;

  $self->make_log_dir( $self->qc_path() );
  my $qc_out = $self->qc_path() . q{/log};
  my $out_subscript = q{.%I.%J.out};
  my $outfile = File::Spec->catfile($qc_out, $job_name . $out_subscript);

  if (defined $indexed) {

    # check that each lane_tag pair can actually have the check run on them,
    # and collate all those together
    $self->add_all_lane_tag_pairs();
    my @lane_tag_pairs = $self->all_lane_tag_pairs();

    $self->clear_lane_tag_pairs();
    my @lanes = $self->positions($arg_refs);
    foreach my $lane_tag_pair ( @lane_tag_pairs ) {

      if (
        ( any { $lane_tag_pair->{position} == $_ } @lanes )
          &&
        $self->_can_run( $lane_tag_pair->{position}, $lane_tag_pair->{tag_index} )
       ) {
        $self->add_lane_tag_pair( $lane_tag_pair );
      }

    }

    if ( $self->has_no_lane_tag_pairs() ) {
      $self->log( q{no indexes found - will not attempt to submit jobs} );
      return;
    }

    $job_name = q{'} . $job_name . $self->lsf_job_array_from_lane_tag_pairs();
    if ( ! $self->no_array_cpu_limit() ) {
       $job_name .= q{%} . $self->array_cpu_limit();
    }
    $job_name .= q{'};
  } else {

    $job_name = q{'} . $job_name . $array_string;
    if ( ! $self->no_array_cpu_limit() ) {
       $job_name .= q{%} . $self->array_cpu_limit();
    }
    $job_name .= q{'};

  }
  #my $job_sub = q{bsub -P pipeline } . $self->farm_options() . qq{ $required_job_completion -J $job_name -o $outfile '$solexa_bin/$QC_SCRIPT --check=}.$self->qc_to_run().q{ --archive_path=}.$self->archive_path().qq{ --position=$position'};

  my $job_sub = q{bsub -q } . $self->lsf_queue() . q{ } . $self->farm_options() . qq{ $required_job_completion -J $job_name -o $outfile};
  if ( ! $NO_REFERENCES_OR_ADAPTER_REQUIREMENTS->{ $self->qc_to_run() } ) {
    $job_sub .= q{ } . $self->ref_adapter_pre_exec_string();
  }
  $job_sub .= q{ '};

  if ( $self->qc_to_run() ne q{cache} ) {

    $job_sub .= File::Spec->catfile( $solexa_bin, $self->external_script_names_conf()->{qc_script} );
    $job_sub .= q{ --check=} . $self->qc_to_run();
    $job_sub .= q{ --archive_path=} . $self->archive_path();
    $job_sub .= q{ --id_run=} . $self->id_run();

    if ( defined $indexed ) {
      $job_sub .= q{ --tag_index=} . $self->tag_index_decode_string();
      $job_sub .= q{ --position=}  . $self->position_decode_string();
    } else {
      $job_sub .= q{ --position=} . $self->lsb_jobindex();
      if ($arg_refs->{$FILTERED_OUT_SPIKED_PHIX}) {
        $job_sub .= q{ } . $arg_refs->{spiked_phix_sequence};
      }
    }

  } else {

    my $number_of_reads = $self->is_paired_read() && $self->is_indexed() ? $NUMBER_OF_READS_INDEXED_AND_PAIRED
                        : $self->is_paired_read() || $self->is_indexed() ? $NUMBER_OF_READS_INDEXED_OR_PAIRED
                        :                                                  $MINIMUM_NUMBER_OF_READS
                        ;

    $job_sub .= q{perl -e "use lib qw[} . $self->filesystem_locations_conf()->{perl5lib} .q{]; use npg_common::extractor::fastq qw[generate_cache]; generate_cache(};
    if ( defined $indexed ) {

      my $id_run_lane_part = $self->archive_path() . q{/lane} . $self->position_decode_string() . q{/} . $self->id_run() . q{_} . $self->position_decode_string();

      $job_sub .= q{ q[} . $self->archive_path() . q{/lane} . $self->position_decode_string() . q(], [ qw{ );

      $job_sub .= $id_run_lane_part . q{_1#} . $self->tag_index_decode_string() . q{.fastq };

      if ( $self->is_paired_read() ) {
        $job_sub .= $id_run_lane_part . q{_2#} . $self->tag_index_decode_string() . q{.fastq };
      }

      $job_sub .= q(} ]);

    } else {

      my $id_run_lane_part = $self->archive_path() . q{/} . $self->id_run() . q{_} . $self->lsb_jobindex();

      $job_sub .= q{ q[} . $self->archive_path() . q(], [ qw{ );

      if ( $number_of_reads == $MINIMUM_NUMBER_OF_READS ) {

        $job_sub .= $id_run_lane_part . q{.fastq };

      } else {

        $job_sub .= $id_run_lane_part . q{_1.fastq };

        if ( $self->is_paired_read() ) {
          $job_sub .= $id_run_lane_part . q{_2.fastq };
        }

        if ( $self->is_indexed() ) {
          $job_sub .= $id_run_lane_part . q{_t.fastq };
        }

      }
      $job_sub .= q(} ]);
    }

    $job_sub .= q{ );"};

  }

  $job_sub .= q{'};

  if ($self->verbose()) { $self->log($job_sub); }

  return $job_sub;
}

##################
# see if the test should be run

sub _can_run {
  my ($self, $position, $tag_index) = @_;
  my $return_value;

  if ( $self->qc_to_run() eq q{cache} ) {
    return 1;
  }

  # if the lane is an indexed lane, then do not run the qc_bias test
  if ( ! defined $tag_index
        &&
       $self->qc_to_run() eq q{gc_bias}
        &&
       ( any { $position == $_ } @{ $self->_lanes_by_split_nonsplit()->{indexed_lanes} } )
  ) {
    return 0;
  }

  my $checks = npg_qc::autoqc::autoqc->checks_list(); # provides an arrayref list of valid checks
  my $check_exists = 0;
  foreach my $check_name ( @{$checks} ) {
    if ( $check_name eq $self->qc_to_run() ) {
      $check_exists = 1;
      my $msg = qq{$check_name exists. Attempting to find out if this is legitimate to run on lane $position};
      if ( defined $tag_index ) {
        $msg .= qq{, tag index $tag_index};
      }
      $self->log( $msg );
      last;
    }
  }

  if ( ! $check_exists ) {
    $self->log( $self->qc_to_run() . qq{ does not exist. No qc jobs will be submitted for lane $position} );
    return 0;
  }

  my $init_hash = {
      archive_path => $self->archive_path(),
      position     => $position,
      check        => $self->qc_to_run(),
      id_run       => $self->id_run(),
  };
  if ( defined $tag_index ) {
      $init_hash->{tag_index} = $tag_index;
  }

  eval {
    my $object = npg_qc::autoqc::autoqc->new($init_hash);
    $return_value = $object->can_run();
    1;
  } or do {
    $self->log($EVAL_ERROR);
    $return_value = 1;
  };

  return $return_value;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::qc

=head1 VERSION

$LastChangedRevision: 13506 $

=head1 SYNOPSIS

  my @job_ids;
  eval {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      qc_to_run => q{test},
#      verbose => 1, # use if you want logging of the commands sent to LSF
    });
    my $arg_refs = {
      required_job_completion  => q{-w'done(123) && done(321)'},
      timestamp                => q{20090709-123456},
      solexa_bin               => q{/software/solexa/bin},
      id_run                   => 1234,
    }

    @job_ids = $aqc->run_qc($arg_refs);
  } or do {
    # your error handling here
  };

=head1 DESCRIPTION

Object module responsible for launching jobs to LSF for qc'ing fastq/fastqcheck files generated from illumina runs

=head1 SUBROUTINES/METHODS

=head2 run_qc - handler to submit perform the operation to launch the qc jobs

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item File::Spec

=item npg_qc::autoqc::autoqc

=item npg_common::roles::run::lane::tag_info

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
