#############
# $Id: base.pm 13506 2011-07-01 09:19:27Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-09-25
# Last Changed On: $Date: 2011-07-01 10:19:27 +0100 (Fri, 01 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-28.0/lib/npg_pipeline/base.pm $

package npg_pipeline::base;
use Moose;
use Carp qw {carp croak cluck confess};
use English qw{-no_match_vars};
use POSIX qw(strftime);
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13506 $ =~ /(\d+)/mxs; $r; };

with qw{MooseX::Getopt
        MooseX::AttributeCloner
        npg_common::roles::log
        npg_common::roles::run::short_info
        npg_pipeline::roles::generic::base
        npg_pipeline::roles::business::base
        npg_pipeline::roles::business::internal_info
        npg_common::roles::run::path_info
      };
with qw{
        npg_common::roles::run::fs_resource
      };

with q{npg_pipeline::roles::business::flag_options};

Readonly::Scalar our $DEFAULT_JOB_ID_FOR_NO_BSUB => 50;
=head1 NAME

npg_pipeline::base

=head1 VERSION

$LastChangedRevision: 13506 $

=head1 SYNOPSIS

In the derived class

  use Moose;
  extends qw{npg_pipeline::base};

Create derived class object

  $oDerived = npg_pipeline::derived->new()

=head1 DESCRIPTION

A base class to provide the following methods to any derived objects within npg_pipeline

=head1 SUBROUTINES/METHODS

=head2 submit_bsub_command - deals with submitting a command to LSF, retrying upto 5 times if the return code is not 0. It will then croak if it still can't submit

  my $LSF_output = $oDerived->submit_bsub_command($cmd);

Note: If the no_bsub flag is set, then this does not submit a job, but instead logs the command, and returns '50' as a job id

=cut

sub submit_bsub_command {
  my ($self, $cmd) = @_;

  if ( $self->has_job_priority() ) {
    my $priority_option = q{-sp } . $self->job_priority();
    $cmd =~ s/bsub/bsub $priority_option/xms;
  }

  # if this job is being submitted from another job which is already running
  # on LSF, then add the current jobs id as a dependency if it doesn't already
  # have dependencies (match [ ]-w)
  if ( $cmd !~ m/[ ]-w/xms && $ENV{LSB_JOBID} ) {
    $cmd =~ s/bsub/bsub -w'done($ENV{LSB_JOBID})'/xms;
  }

  # add job_name_prefix into command
  # we assume that the first -J is to do with the bsub command, any extra will be in the main command, and we don't want to lose this
  my ( $bsub_before_job_name, $jobs_name_plus, @any_extra ) = split /-J/xms, $cmd;
  my $job_name_prefix = $self->job_name_prefix();

  my ( $whitespace ) = $jobs_name_plus =~ /\A(\s+)/xms;
  $jobs_name_plus =~ s/\A\s+//xms;
  $whitespace ||= q{};
  my ( $quote ) = $jobs_name_plus =~ /\A(')/xms;
  $jobs_name_plus =~ s/\A'//xms;
  $quote ||= q{};
  $jobs_name_plus = $whitespace . $quote . $job_name_prefix . $jobs_name_plus;
  $cmd = join '-J', $bsub_before_job_name, $jobs_name_plus, @any_extra;

  # if the no_bsub flag is set
  if ( $self->no_bsub() ) {
    $self->log( q{*************} );
    $self->log( q{I would be submitting the following to LSF} );
    $self->log( $cmd );
    $self->log( q{*************} );
    return $DEFAULT_JOB_ID_FOR_NO_BSUB; # returns a number so that it can be propagated through to other jobs as a dependency if applicable
  }

  my $count = 1;
  my $job_id;

  $cmd = $self->_test_bin() . $cmd; # in the test suite, prepends t/bin/ to the bsub command

  my $max_tries_plus_one = $self->general_values_conf()->{max_tries} + 1;
  my $min_sleep = $self->general_values_conf()->{min_sleep};

  while ($count < $max_tries_plus_one) {
    $job_id = qx/$cmd/;
    if ($CHILD_ERROR) {
      $self->log(qq{Error attempting ($count) to submit job $cmd \n\n.\tError code $CHILD_ERROR});
      sleep $min_sleep ** $count;
      $count++;
    } else {
      $count = $max_tries_plus_one;
    }
  }
  if ( $job_id !~ /\d+/xms ) {
    croak qq{Failed to submit an lsf job for $cmd};
  }
  return $job_id;
}

###### private methods

# this is just a simple internal private method so that a test bin can be applied to a command
has q{_test_bin} => (isa => q{Str}, is => q{ro}, lazy_build => 1);

sub _build__test_bin {
  my ($self) = @_;
  my $test_bin = $ENV{TEST_BIN} || q{};
  return $test_bin;
}

# this class ties together short_info and path_info, so the following _build_run_folder will work
sub _build_run_folder {
  my ($self) = @_;
  my @temp = split m{/}xms, $self->runfolder_path();
  my $run_folder = pop @temp;
  return $run_folder;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly
=item MooseX::Getopt
=item MooseX::AttributeCloner
=item npg_common::roles::log
=item npg_common::roles::run::short_info

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
