#############
# $Id: harold_calibration_reqs.pm 13506 2011-07-01 09:19:27Z ajb $
# Created By: 
# Last Maintained By: $Author: ajb $
# Created On: 
# Last Changed On: $Date: 2011-07-01 10:19:27 +0100 (Fri, 01 Jul 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-28.0/lib/npg_pipeline/roles/business/harold_calibration_reqs.pm $

package npg_pipeline::roles::business::harold_calibration_reqs;
use Moose::Role;
use English qw{-no_match_vars};
use Carp qw{cluck confess carp croak};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13506 $ =~ /(\d+)/mxs; $r; };
use npg_common::sequence::reference;

requires qw{directory_exists log pick_control_lane};

# hard-coded default parameters for running harold_calibration steps, can be overriden on the command line
# whilst these can be overridden, they are requested to be how the pipeline operates, so we don't
# want them in a config file
Readonly::Scalar our $CAL_TABLE_JOB             => q{PB_cal_table};
Readonly::Scalar our $SCORE_JOB                 => q{PB_cal_score};
Readonly::Scalar our $ALIGN_JOB                 => q{PB_cal_align};
Readonly::Scalar our $PB_DIRECTORY              => q{PB_cal};

=head1 NAME

npg_pipeline::roles::business::harold_calibration_reqs

=head1 VERSION

$LastChangedRevision: 13506 $

=head1 SYNOPSIS

  package MyPackage;
  use Moose;
  ...
  with qw{npg_pipeline::roles::business::harold_calibration_reqs};

=head1 DESCRIPTION

This role is designed to be able to apply all the harold calibration variables, including lsf requirements,
for internal running of the harold calibration steps.

Note, your class must provide the following methods

 'log'
 'directory_exists'
 'pick_control_lane'

=head1 SUBROUTINES/METHODS

=head2 pb_calibration_bin

returns the location of the pb_calibration scripts

  my $sPBCalibrationBin = $class->pb_calibration_bin();

=cut

has q{pb_calibration_bin} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
  documentation => q{bin used for pb_calibration scripts},);

sub _build_pb_calibration_bin {
  my ( $self ) = @_;
  return $self->filesystem_locations_conf()->{src} . q{/} . $self->pb_cal_pipeline_conf()->{bin};
}

has q{random} => (isa => q{Int}, is => q{ro}, lazy_build => 1,
  documentation => q{Default from pb_cal_pipeline.ini},);

sub _build_random {
  my ( $self ) = @_;
  return $self->pb_cal_pipeline_conf()->{random};
}

has q{t_filter} => (isa => q{Int}, is => q{ro}, lazy_build => 1,
  documentation => q{t_filter value},);

sub _build_t_filter {
  my ( $self ) = @_;
  return $self->pb_cal_pipeline_conf()->{t_filter};
}

has q{mem_calibration} => (isa => q{Int}, is => q{ro}, lazy_build => 1,
  documentation => q{memory to be used for calibration table creation jobs},);

sub _build_mem_calibration {
  my ( $self ) = @_;
  return $self->pb_cal_pipeline_conf()->{mem_calibration};
}

has q{mem_score} => (isa => q{Int}, is => q{ro}, lazy_build => 1,
  documentation => q{memory to be used for scoring jobs},);

sub _build_mem_score {
  my ( $self ) = @_;
  return $self->pb_cal_pipeline_conf()->{mem_score};
}

has q{cal_table_job} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
  documentation => qq{Default : $CAL_TABLE_JOB},);

sub _build_cal_table_job { return $CAL_TABLE_JOB; }

has q{cal_table_script} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
  documentation => q{Default from pb_cal_pipeline.ini},);

sub _build_cal_table_script {
  my ( $self ) = @_;
  return $self->pb_cal_pipeline_conf()->{cal_table_script};
}

has q{align_job} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
  documentation => qq{Default : $ALIGN_JOB},);

sub _build_align_job { return $ALIGN_JOB; }

has q{alignment_script} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
  documentation => q{Default from pb_cal_pipeline.ini},);

sub _build_alignment_script {
  my ( $self ) = @_;
  return $self->pb_cal_pipeline_conf()->{alignment_script};
}

has q{recalibration_script} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
  documentation => q{Default from pb_cal_pipeline.ini},);

sub _build_recalibration_script {
  my ( $self ) = @_;
  return $self->pb_cal_pipeline_conf()->{recalibration_script};
}

has q{pb_directory} => (isa => q{Str}, is => q{ro}, lazy_build => 1, init_arg => undef);

sub _build_pb_directory { return $PB_DIRECTORY; }

has q{score_job} => (isa => q{Str}, is => q{ro}, lazy_build => 1,
  documentation => qq{Default : $SCORE_JOB},);

sub _build_score_job { return $SCORE_JOB; }

has q{pb_cal_mode} => (isa => q{Int}, is => q{ro}, lazy_build => 1,
  documentation => q{Default in pb_cal_pipeline.ini},);

sub _build_pb_cal_mode {
  my ( $self ) = @_;
  return $self->pb_cal_pipeline_conf()->{default_mode};
}

=head2 calibration_table_name

generates the calibration table name expected, requiring the id_run and read to be passes in
if no control lane can be worked out, will return an empty string

  my $sCalibrationTableName = $class->calibration_table_name( $iIdRun, $iRead );

=cut

sub calibration_table_name {
  my ($self, $arg_refs ) = @_;
  my $id_run = $arg_refs->{id_run};
  my $read = $arg_refs->{read};
  my $position = $arg_refs->{position};
  # set the mode
  my $mode = $arg_refs->{mode} || $self->pb_cal_pipeline_conf()->{default_mode};

  # if no position is given, we want the control lane
  # if the position is the control lane, then we want the mode for that
  # unless user defined
  if ( ! $position || $position == $self->pick_control_lane() ) {
    $position = $self->pick_control_lane();
    $mode = $self->pb_cal_pipeline_conf()->{control_mode};
  }

  if ( ! $position ) {
    $self->log( q{no position obtained} );
    return q{};
  }

  # finally, overwrite mode to user defined mode if that has been set
  if ( $self->has_pb_cal_mode() ) {
    $mode = $self->pb_cal_mode();
  }

  return $id_run . q{_} . $position . q{_} . $read . $self->_cal_table_suffixes()->{$mode};
}

# cal table suffixes based on mode
has q{_cal_table_suffixes} => (
  isa => q{HashRef},
  is  => q{ro},
  lazy_build => 1,
);

sub _build__cal_table_suffixes {
  my ( $self ) = @_;

  return {
    1 => $self->pb_cal_pipeline_conf()->{cal_table_suffix},  ## no critic (ValuesAndExpressions::ProhibitCommaSeparatedStatements)
    2 => $self->pb_cal_pipeline_conf()->{mode2_cal_table_suffix},
  };
}

=head2 create_pb_calibration_directory

checks for the existence of a pb_calibration directory and if it doesn't exist, will create it

returns the path of the pb_calibration directory

=cut

sub create_pb_calibration_directory {
  my ( $self ) = @_;

  my $pb_cal_dir = $self->pb_cal_path();

  if ( ! $self->directory_exists( $pb_cal_dir ) ) {
    $self->log( qq{Creating $pb_cal_dir} );

    my $output = qx[mkdir $pb_cal_dir];
    if ($CHILD_ERROR) {
      croak qq{Unable to create $pb_cal_dir};
    }

    $self->log( qq{Created : $output});
  }

  $self->make_log_dir( $pb_cal_dir );

  return $pb_cal_dir;
}

=head2 ref_seq

As long as there should be recalibration occuring, either from a control lane or lane's own reference, this method will
return the information for the lane

You can specify a reference sequence on the command line, and this will be used, assuming that there is a control lane
for it to be applied to.

=cut

has q{_ref_seq} => (isa => q{Str}, is => q{ro}, lazy_build => 1, init_arg => q{ref_seq},
  documentation => q{override any reference for the control lane},);

has q{_spiked_ref_seq} => (isa => q{Str}, is => q{ro}, lazy_build => 1, init_arg => q{spiked_ref_seq},
  documentation => qq{override any reference for the spiked in control, default will be for that in general_values config file,\nwith pb_cal_pipeline config files default aligner},);

sub _build__ref_seq {
  my ( $self ) = @_;
#  my $ref = $self->references_conf()->{phix};
#  return $ref;
  return $self->_spiked_ref_seq();
}

sub _build__spiked_ref_seq {
  my ( $self ) = @_;

  my $arg_refs = {
    aligner => $self->pb_cal_pipeline_conf()->{default_aligner},
    species => $self->general_values_conf()->{spiked_species},
  };

  if ( $self->has_repository() ) {
    $arg_refs->{repository} = $self->repository();
  }

  my $ruser = Moose::Meta::Class->create_anon_class(
        roles => [qw/npg_common::sequence::reference::roles::find/])
        ->new_object($arg_refs);

  my $reference = q{};
  eval {
    $reference = $ruser->refs->[0];
    1;
  } or do{
    carp $EVAL_ERROR;
  };

  return $reference;
}

sub ref_seq {
  my ( $self, $arg_refs ) = @_;

  if ( $self->no_recalibration() ) {
    return;
  }

  if ( $arg_refs->{is_spiked_phix} ) {
    return $self->_spiked_ref_seq();
  }


  if ( $arg_refs->{control} ) {
    return $self->_ref_seq();
  }

  my $ref_seq;

  if ( $arg_refs->{lane} ) {

    my $hash_for_new = $arg_refs->{lane};

    my $id_run    = $hash_for_new->{id_run};
    my $position  = $hash_for_new->{position};

    if ( $self->has_ref_repository() ) {
      $hash_for_new->{ref_repository} = $self->ref_repository();
    }

    my $ref_seeker = npg_common::sequence::reference->new( $hash_for_new );
    my $refs = $ref_seeker->refs();

    # code to obtain and return reference sequence based on id_run and lane position

    if ( ! $ref_seeker->single_ref_found() ) {
      my $log_string = q{Could not find a 'single' reference for } . $id_run . q{:} . $position . qq{\n};

      if ( ! $refs ) {
        $log_string .= qq{\tNothing returned at all}
      } elsif ( ! scalar @{ $refs } ) {
        $log_string .= qq{\tno reference sequences found.};
      } else {
        $log_string .= qq{\tmore than one reference found.\n\t\t};
        $log_string .= ( join qq{\n\t\t}, @{ $refs } );
      }

      $self->log( $log_string );

      return;
    }

    $ref_seq = $refs->[0];

  }

  return $ref_seq;
} # currently now optimised to use the ref seq finder. When ready, should actually utilise the cached data

1;
__END__
          
=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
